/*
 * Copyright (C) 2002-2009 Keisuke Nishida
 * Copyright (C) 2007-2009 Roger While
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public License
 * as published by the Free Software Foundation; either version 2.1,
 * or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; see the file COPYING.LIB.  If
 * not, write to the Free Software Foundation, 51 Franklin Street, Fifth Floor
 * Boston, MA 02110-1301 USA
 */

#ifndef COB_LIBCOB_H
#define COB_LIBCOB_H

#include <gmp.h>

#ifdef __cplusplus
extern "C" {
#endif

#include <libcob/byteswap.h>
#include <libcob/call.h>
#include <libcob/common.h>
#include <libcob/fileio.h>
#include <libcob/move.h>
#include <libcob/numeric.h>
#include <libcob/screenio.h>
#include <libcob/strings.h>
#include <libcob/termio.h>
#include <libcob/intrinsic.h>
#include <libcob/codegen.h>

#ifdef __cplusplus
}
#endif

#endif /* COB_LIBCOB_H_ */
